"""
This file is a translation from Objective-C. The original
copy-right:

//  Copyright 2009 Matt Gallagher. All rights reserved.
//
//  Permission is given to use this source code file, free of charge, in any
//  project, commercial or otherwise, entirely at your risk, with the condition
//  that any redistribution (in part or whole) of source code must retain
//  this copyright and permission notice. Attribution in compiled projects is
//  appreciated but not required.
"""

import math

import Cocoa
import WebKit  # noqa: F401
import CoreLocation
import objc


class WhereIsMyMacAppDelegate(Cocoa.NSObject):
    window = objc.ivar()
    webView = objc.IBOutlet()
    locationManager = objc.ivar()
    locationLabel = objc.IBOutlet()
    accuracyLabel = objc.IBOutlet()

    def applicationDidFinishLaunching_(self, notification):
        self.locationManager = CoreLocation.CLLocationManager.alloc().init()
        self.locationManager.setDelegate_(self)
        self.locationManager.startUpdatingLocation()

    @classmethod
    def latitudeRangeForLocation_(self, location):
        M = 6_367_000.0
        # approximate average meridional radius of curvature of earth
        metersToLatitude = 1.0 / ((math.pi / 180.0) * M)
        accuracyToWindowScale = 2.0

        return location.horizontalAccuracy() * metersToLatitude * accuracyToWindowScale

    @classmethod
    def longitudeRangeForLocation_(self, location):
        latitudeRange = WhereIsMyMacAppDelegate.latitudeRangeForLocation_(location)

        return latitudeRange * math.cos(
            location.coordinate().latitude * math.pi / 180.0
        )

    @objc.IBAction
    def openInDefaultBrowser_(self, sender):
        currentLocation = self.locationManager.location()

        externalBrowserURL = Cocoa.NSURL.URLWithString_(
            "http://maps.google.com/maps?ll=%f,%f&amp;spn=%f,%f"
            % (
                currentLocation.coordinate().latitude,
                currentLocation.coordinate().longitude,
                WhereIsMyMacAppDelegate.latitudeRangeForLocation_(currentLocation),
                WhereIsMyMacAppDelegate.longitudeRangeForLocation_(currentLocation),
            )
        )

        Cocoa.NSWorkspace.sharedWorkspace().openURL_(externalBrowserURL)

    def locationManager_didUpdateToLocation_fromLocation_(
        self, manager, newLocation, oldLocation
    ):

        # Ignore updates where nothing we care about changed
        if newLocation is None:
            return
        if oldLocation is None:
            pass
        elif (
            newLocation.coordinate().longitude == oldLocation.coordinate().longitude
            and newLocation.coordinate().latitude == oldLocation.coordinate().latitude
            and newLocation.horizontalAccuracy() == oldLocation.horizontalAccuracy()
        ):
            return

        # Load the HTML for displaying the Google map from a file and replace the
        # format placeholders with our location data
        path = Cocoa.NSBundle.mainBundle().pathForResource_ofType_(
            "HTMLFormatString", "html"
        )
        with open(path) as fp:
            htmlString = fp.read() % (
                newLocation.coordinate().latitude,
                newLocation.coordinate().longitude,
                WhereIsMyMacAppDelegate.latitudeRangeForLocation_(newLocation),
                WhereIsMyMacAppDelegate.longitudeRangeForLocation_(newLocation),
            )

        # Load the HTML in the WebView and set the labels
        self.webView.mainFrame().loadHTMLString_baseURL_(htmlString, None)
        self.locationLabel.setStringValue_(
            "%f, %f"
            % (newLocation.coordinate().latitude, newLocation.coordinate().longitude)
        )
        self.accuracyLabel.setStringValue_(f"{newLocation.horizontalAccuracy():f}")

    def locationManager_didFailWithError_(self, manager, error):
        self.webView.mainFrame.loadHTMLString_baseURL_(
            Cocoa.NSLocalizedString("Location manager failed with error: %s", objc.nil)
            % (error.localizedDescription()),
            None,
        )
        self.locationLabel.setStringValue_("")
        self.accuracyLabel.setStringValue_("")

    def applicationWillTerminate_(self, aNotification):
        self.locationManager.stopUpdatingLocation()



import os

import CoreLocation
from PyObjCTools.TestSupport import (
    TestCase,
    min_os_level,
    os_release,
    os_level_key,
    expectedFailureIf,
)


class TestCLError(TestCase):
    @min_os_level("10.6")
    def testConstants(self):
        self.assertIsInstance(CoreLocation.kCLErrorDomain, str)

        self.assertEqual(CoreLocation.kCLErrorLocationUnknown, 0)
        self.assertEqual(CoreLocation.kCLErrorDenied, 1)
        self.assertEqual(CoreLocation.kCLErrorNetwork, 2)
        self.assertEqual(CoreLocation.kCLErrorHeadingFailure, 3)
        self.assertEqual(CoreLocation.kCLErrorRegionMonitoringDenied, 4)
        self.assertEqual(CoreLocation.kCLErrorRegionMonitoringFailure, 5)
        self.assertEqual(CoreLocation.kCLErrorRegionMonitoringSetupDelayed, 6)
        self.assertEqual(CoreLocation.kCLErrorRegionMonitoringResponseDelayed, 7)
        self.assertEqual(CoreLocation.kCLErrorGeocodeFoundPartialResult, 9)
        self.assertEqual(CoreLocation.kCLErrorDeferredFailed, 11)
        self.assertEqual(CoreLocation.kCLErrorDeferredNotUpdatingLocation, 12)
        self.assertEqual(CoreLocation.kCLErrorDeferredAccuracyTooLow, 13)
        self.assertEqual(CoreLocation.kCLErrorDeferredDistanceFiltered, 14)
        self.assertEqual(CoreLocation.kCLErrorDeferredCanceled, 15)
        self.assertEqual(CoreLocation.kCLErrorRangingUnavailable, 16)
        self.assertEqual(CoreLocation.kCLErrorRangingFailure, 17)
        self.assertEqual(CoreLocation.kCLErrorPromptDeclined, 18)

        if int(os.uname()[2].split(".")[0]) < 12:
            self.assertEqual(CoreLocation.kCLErrorGeocodeFoundNoResult, 7)
            self.assertEqual(CoreLocation.kCLErrorGeocodeCanceled, 8)
        else:
            self.assertEqual(CoreLocation.kCLErrorGeocodeFoundNoResult, 8)
            self.assertEqual(CoreLocation.kCLErrorGeocodeCanceled, 10)

    @min_os_level("10.7")
    @expectedFailureIf(os_level_key(os_release()) < os_level_key("10.13"))
    def testConstants10_7(self):
        self.assertIsInstance(CoreLocation.kCLErrorUserInfoAlternateRegionKey, str)



"""
Python mapping for the CoreLocation framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""

import os
import sys

import Foundation
import objc
from CoreLocation import _metadata
from CoreLocation import _CoreLocation

if int(os.uname()[2].split(".")[0]) < 12:
    # OSX <= 10.7
    kCLErrorGeocodeFoundNoResult = 7
    kCLErrorGeocodeCanceled = 8
else:
    # OSX 10.8 or later
    kCLErrorGeocodeFoundNoResult = 8
    kCLErrorGeocodeCanceled = 10

sys.modules["CoreLocation"] = mod = objc.ObjCLazyModule(
    "CoreLocation",
    "com.apple.corelocation",
    objc.pathForFramework("/System/Library/Frameworks/CoreLocation.framework"),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
        "objc": objc,
        "kCLErrorGeocodeFoundNoResult": kCLErrorGeocodeFoundNoResult,
        "kCLErrorGeocodeCanceled": kCLErrorGeocodeCanceled,
    },
    (_CoreLocation, Foundation),
)


del sys.modules["CoreLocation._metadata"]



import CoreLocation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCLLocationManager(TestCase):
    @min_os_level("10.6")
    def testMethods(self):
        self.assertResultIsBOOL(
            CoreLocation.CLLocationManager.alloc().init().locationServicesEnabled
        )

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(CoreLocation.CLDeviceOrientationUnknown, 0)
        self.assertEqual(CoreLocation.CLDeviceOrientationPortrait, 1)
        self.assertEqual(CoreLocation.CLDeviceOrientationPortraitUpsideDown, 2)
        self.assertEqual(CoreLocation.CLDeviceOrientationLandscapeLeft, 3)
        self.assertEqual(CoreLocation.CLDeviceOrientationLandscapeRight, 4)
        self.assertEqual(CoreLocation.CLDeviceOrientationFaceUp, 5)
        self.assertEqual(CoreLocation.CLDeviceOrientationFaceDown, 6)

        self.assertEqual(CoreLocation.kCLAuthorizationStatusNotDetermined, 0)
        self.assertEqual(CoreLocation.kCLAuthorizationStatusRestricted, 1)
        self.assertEqual(CoreLocation.kCLAuthorizationStatusDenied, 2)
        self.assertEqual(CoreLocation.kCLAuthorizationStatusAuthorizedAlways, 3)
        self.assertEqual(
            CoreLocation.kCLAuthorizationStatusAuthorized,
            CoreLocation.kCLAuthorizationStatusAuthorizedAlways,
        )

        self.assertEqual(CoreLocation.CLActivityTypeOther, 1)
        self.assertEqual(CoreLocation.CLActivityTypeAutomotiveNavigation, 2)
        self.assertEqual(CoreLocation.CLActivityTypeFitness, 3)
        self.assertEqual(CoreLocation.CLActivityTypeOtherNavigation, 4)
        self.assertEqual(CoreLocation.CLActivityTypeAirborne, 5)

        self.assertEqual(CoreLocation.CLAccuracyAuthorizationFullAccuracy, 0)
        self.assertEqual(CoreLocation.CLAccuracyAuthorizationReducedAccuracy, 1)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(CoreLocation.CLLocationManager.locationServicesEnabled)
        self.assertResultIsBOOL(CoreLocation.CLLocationManager.headingAvailable)
        self.assertResultIsBOOL(
            CoreLocation.CLLocationManager.significantLocationChangeMonitoringAvailable
        )
        self.assertResultIsBOOL(
            CoreLocation.CLLocationManager.regionMonitoringAvailable
        )
        self.assertResultIsBOOL(CoreLocation.CLLocationManager.regionMonitoringEnabled)

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertResultIsBOOL(
            CoreLocation.CLLocationManager.deferredLocationUpdatesAvailable
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(
            CoreLocation.CLLocationManager.isMonitoringAvailableForClass_
        )
        self.assertResultIsBOOL(CoreLocation.CLLocationManager.locationServicesEnabled)
        self.assertResultIsBOOL(
            CoreLocation.CLLocationManager.pausesLocationUpdatesAutomatically
        )
        self.assertArgIsBOOL(
            CoreLocation.CLLocationManager.setPausesLocationUpdatesAutomatically_, 0
        )
        self.assertResultIsBOOL(
            CoreLocation.CLLocationManager.allowsBackgroundLocationUpdates
        )
        self.assertArgIsBOOL(
            CoreLocation.CLLocationManager.setAllowsBackgroundLocationUpdates_, 0
        )
        self.assertResultIsBOOL(CoreLocation.CLLocationManager.headingAvailable)

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertResultIsBOOL(CoreLocation.CLLocationManager.isRangingAvailable)

    @min_os_level("11.0")
    def testMethods11_0(self):
        self.assertResultIsBOOL(
            CoreLocation.CLLocationManager.isAuthorizedForWidgetUpdates
        )
        self.assertResultIsBOOL(
            CoreLocation.CLLocationManager.isAuthorizedForPreciseLocation
        )

        self.assertArgIsBlock(
            CoreLocation.CLLocationManager.requestTemporaryFullAccuracyAuthorizationWithPurposeKey_completion_,
            1,
            b"v@",
        )
        self.assertArgIsBlock(
            CoreLocation.CLLocationManager.requestTemporaryPreciseLocationAuthorizationWithPurposeKey_completion_,
            1,
            b"v@",
        )



import CoreLocation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCLRegion(TestCase):
    def testConstants(self):
        self.assertEqual(CoreLocation.CLRegionStateUnknown, 0)
        self.assertEqual(CoreLocation.CLRegionStateInside, 1)
        self.assertEqual(CoreLocation.CLRegionStateOutside, 2)

        self.assertEqual(CoreLocation.CLProximityUnknown, 0)
        self.assertEqual(CoreLocation.CLProximityImmediate, 1)
        self.assertEqual(CoreLocation.CLProximityNear, 2)
        self.assertEqual(CoreLocation.CLProximityFar, 3)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(CoreLocation.CLRegion.containsCoordinate_)
        self.assertArgHasType(
            CoreLocation.CLRegion.containsCoordinate_,
            0,
            CoreLocation.CLLocationCoordinate2D.__typestr__,
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(CoreLocation.CLRegion.notifyOnEntry)
        self.assertArgIsBOOL(CoreLocation.CLRegion.setNotifyOnEntry_, 0)
        self.assertResultIsBOOL(CoreLocation.CLRegion.notifyOnExit)
        self.assertArgIsBOOL(CoreLocation.CLRegion.setNotifyOnExit_, 0)



import CoreLocation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCLHeading(TestCase):
    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(CoreLocation.kCLHeadingFilterNone, float)



import CoreLocation
from PyObjCTools.TestSupport import TestCase
import objc


class TestCLLocationManagerDelegateHelper(CoreLocation.NSObject):
    def locationManagerShouldDisplayHeadingCalibration_(self, manager):
        return 1

    def locationManager_didDetermineState_forRegion_(self, m, s, r):
        pass

    def locationManager_didChangeAuthorizationStatus_(self, m, s):
        pass


class TestCLLocationManagerDelegate(TestCase):
    def testProtocols(self):
        objc.protocolNamed("CLLocationManagerDelegate")

        self.assertResultIsBOOL(
            TestCLLocationManagerDelegateHelper.locationManagerShouldDisplayHeadingCalibration_
        )
        self.assertArgHasType(
            TestCLLocationManagerDelegateHelper.locationManager_didDetermineState_forRegion_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestCLLocationManagerDelegateHelper.locationManager_didChangeAuthorizationStatus_,
            1,
            objc._C_INT,
        )



import CoreLocation
from PyObjCTools.TestSupport import TestCase, min_os_level

CLGeocodeCompletionHandler = b"v@@"


class TestCLGeocoder(TestCase):
    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertArgIsBlock(
            CoreLocation.CLGeocoder.geocodeAddressDictionary_completionHandler_,
            1,
            CLGeocodeCompletionHandler,
        )
        self.assertArgIsBlock(
            CoreLocation.CLGeocoder.geocodeAddressString_completionHandler_,
            1,
            CLGeocodeCompletionHandler,
        )
        self.assertArgIsBlock(
            CoreLocation.CLGeocoder.geocodeAddressString_inRegion_completionHandler_,
            2,
            CLGeocodeCompletionHandler,
        )
        self.assertArgIsBlock(
            CoreLocation.CLGeocoder.reverseGeocodeLocation_completionHandler_,
            1,
            CLGeocodeCompletionHandler,
        )

        self.assertResultIsBOOL(CoreLocation.CLGeocoder.isGeocoding)

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBlock(
            CoreLocation.CLGeocoder.reverseGeocodeLocation_preferredLocale_completionHandler_,
            2,
            CLGeocodeCompletionHandler,
        )
        self.assertArgIsBlock(
            CoreLocation.CLGeocoder.geocodeAddressString_inRegion_preferredLocale_completionHandler_,
            3,
            CLGeocodeCompletionHandler,
        )
        self.assertArgIsBlock(
            CoreLocation.CLGeocoder.geocodePostalAddress_completionHandler_,
            1,
            CLGeocodeCompletionHandler,
        )
        self.assertArgIsBlock(
            CoreLocation.CLGeocoder.geocodePostalAddress_preferredLocale_completionHandler_,
            2,
            CLGeocodeCompletionHandler,
        )



import CoreLocation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCLCircularRegion(TestCase):
    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(CoreLocation.CLCircularRegion.containsCoordinate_)
        self.assertArgHasType(
            CoreLocation.CLCircularRegion.containsCoordinate_,
            0,
            CoreLocation.CLLocationCoordinate2D.__typestr__,
        )

        self.assertResultHasType(
            CoreLocation.CLCircularRegion.center,
            CoreLocation.CLLocationCoordinate2D.__typestr__,
        )



"""
Wrappers for framework 'CoreLocation' on macOS 10.6. This framework provides
an interface for dealing with the physical location of a machine, which allows
for geo-aware applications.

These wrappers don't include documentation, please check Apple's documentation
for information on how to use this framework and PyObjC's documentation
for general tips and tricks regarding the translation between Python
and (Objective-)C frameworks
"""
import os

from pyobjc_setup import Extension, setup

VERSION = "8.3"

setup(
    name="pyobjc-framework-CoreLocation",
    description="Wrappers for the framework CoreLocation on macOS",
    min_os_level="10.6",
    packages=["CoreLocation"],
    ext_modules=[
        Extension(
            "CoreLocation._CoreLocation",
            ["Modules/_CoreLocation.m"],
            extra_link_args=["-framework", "CoreLocation"],
            py_limited_api=True,
            depends=[
                os.path.join("Modules", fn)
                for fn in os.listdir("Modules")
                if fn.startswith("_CoreLocation")
            ],
        )
    ],
    version=VERSION,
    install_requires=["pyobjc-core>=" + VERSION, "pyobjc-framework-Cocoa>=" + VERSION],
    long_description=__doc__,
    options={"bdist_wheel": {"py_limited_api": "cp36"}},
)



# This file is generated by objective.metadata
#
# Last update: Thu Dec  2 21:24:56 2021
#
# flake8: noqa

import objc, sys

if sys.maxsize > 2**32:

    def sel32or64(a, b):
        return b

else:

    def sel32or64(a, b):
        return a


if objc.arch == "arm64":

    def selAorI(a, b):
        return a

else:

    def selAorI(a, b):
        return b


misc = {}
misc.update(
    {
        "CLLocationCoordinate2D": objc.createStructType(
            "CoreLocation.CLLocationCoordinate2D",
            b"{CLLocationCoordinate2D=dd}",
            ["latitude", "longitude"],
        )
    }
)
constants = """$CLLocationDistanceMax@d$CLLocationPushServiceErrorDomain$CLTimeIntervalMax@d$kCLDistanceFilterNone@d$kCLErrorDomain$kCLErrorUserInfoAlternateRegionKey$kCLHeadingFilterNone@d$kCLLocationAccuracyBest@d$kCLLocationAccuracyBestForNavigation@d$kCLLocationAccuracyHundredMeters@d$kCLLocationAccuracyKilometer@d$kCLLocationAccuracyNearestTenMeters@d$kCLLocationAccuracyReduced@d$kCLLocationAccuracyThreeKilometers@d$kCLLocationCoordinate2DInvalid@{CLLocationCoordinate2D=dd}$"""
enums = """$CLAccuracyAuthorizationFullAccuracy@0$CLAccuracyAuthorizationReducedAccuracy@1$CLActivityTypeAirborne@5$CLActivityTypeAutomotiveNavigation@2$CLActivityTypeFitness@3$CLActivityTypeOther@1$CLActivityTypeOtherNavigation@4$CLDeviceOrientationFaceDown@6$CLDeviceOrientationFaceUp@5$CLDeviceOrientationLandscapeLeft@3$CLDeviceOrientationLandscapeRight@4$CLDeviceOrientationPortrait@1$CLDeviceOrientationPortraitUpsideDown@2$CLDeviceOrientationUnknown@0$CLLocationPushServiceErrorMissingEntitlement@3$CLLocationPushServiceErrorMissingPushExtension@1$CLLocationPushServiceErrorMissingPushServerEnvironment@2$CLLocationPushServiceErrorUnknown@0$CLProximityFar@3$CLProximityImmediate@1$CLProximityNear@2$CLProximityUnknown@0$CLRegionStateInside@1$CLRegionStateOutside@2$CLRegionStateUnknown@0$kCLAuthorizationStatusAuthorized@3$kCLAuthorizationStatusAuthorizedAlways@3$kCLAuthorizationStatusAuthorizedWhenInUse@4$kCLAuthorizationStatusDenied@2$kCLAuthorizationStatusNotDetermined@0$kCLAuthorizationStatusRestricted@1$kCLErrorDeferredAccuracyTooLow@13$kCLErrorDeferredCanceled@15$kCLErrorDeferredDistanceFiltered@14$kCLErrorDeferredFailed@11$kCLErrorDeferredNotUpdatingLocation@12$kCLErrorDenied@1$kCLErrorGeocodeFoundPartialResult@9$kCLErrorHeadingFailure@3$kCLErrorLocationUnknown@0$kCLErrorNetwork@2$kCLErrorPromptDeclined@18$kCLErrorRangingFailure@17$kCLErrorRangingUnavailable@16$kCLErrorRegionMonitoringDenied@4$kCLErrorRegionMonitoringFailure@5$kCLErrorRegionMonitoringResponseDelayed@7$kCLErrorRegionMonitoringSetupDelayed@6$"""
misc.update({})
functions = {
    "CLLocationCoordinate2DIsValid": (b"Z{CLLocationCoordinate2D=dd}",),
    "CLLocationCoordinate2DMake": (b"{CLLocationCoordinate2D=dd}dd",),
}
aliases = {"kCLAuthorizationStatusAuthorized": "kCLAuthorizationStatusAuthorizedAlways"}
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(b"CLBeaconRegion", b"notifyEntryStateOnDisplay", {"retval": {"type": b"Z"}})
    r(
        b"CLBeaconRegion",
        b"setNotifyEntryStateOnDisplay:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"CLCircularRegion",
        b"center",
        {"retval": {"type": b"{CLLocationCoordinate2D=dd}"}},
    )
    r(
        b"CLCircularRegion",
        b"containsCoordinate:",
        {
            "retval": {"type": b"Z"},
            "arguments": {2: {"type": b"{CLLocationCoordinate2D=dd}"}},
        },
    )
    r(
        b"CLCircularRegion",
        b"initWithCenter:radius:identifier:",
        {"arguments": {2: {"type": b"{CLLocationCoordinate2D=dd}"}}},
    )
    r(
        b"CLGeocoder",
        b"geocodeAddressDictionary:completionHandler:",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"CLGeocoder",
        b"geocodeAddressString:completionHandler:",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"CLGeocoder",
        b"geocodeAddressString:inRegion:completionHandler:",
        {
            "arguments": {
                4: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"CLGeocoder",
        b"geocodeAddressString:inRegion:preferredLocale:completionHandler:",
        {
            "arguments": {
                5: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"CLGeocoder",
        b"geocodePostalAddress:completionHandler:",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"CLGeocoder",
        b"geocodePostalAddress:preferredLocale:completionHandler:",
        {
            "arguments": {
                4: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(b"CLGeocoder", b"isGeocoding", {"retval": {"type": b"Z"}})
    r(
        b"CLGeocoder",
        b"reverseGeocodeLocation:completionHandler:",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"CLGeocoder",
        b"reverseGeocodeLocation:preferredLocale:completionHandler:",
        {
            "arguments": {
                4: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(b"CLLocation", b"coordinate", {"retval": {"type": "{CLLocationCoordinate2D=dd}"}})
    r(
        b"CLLocation",
        b"initWithCoordinate:altitude:horizontalAccuracy:verticalAccuracy:course:speed:timestamp:",
        {"arguments": {2: {"type": b"{CLLocationCoordinate2D=dd}"}}},
    )
    r(
        b"CLLocation",
        b"initWithCoordinate:altitude:horizontalAccuracy:verticalAccuracy:timestamp:",
        {"arguments": {2: {"type": "{CLLocationCoordinate2D=dd}"}}},
    )
    r(
        b"CLLocationManager",
        b"allowsBackgroundLocationUpdates",
        {"retval": {"type": b"Z"}},
    )
    r(
        b"CLLocationManager",
        b"deferredLocationUpdatesAvailable",
        {"retval": {"type": b"Z"}},
    )
    r(b"CLLocationManager", b"headingAvailable", {"retval": {"type": b"Z"}})
    r(
        b"CLLocationManager",
        b"isAuthorizedForPreciseLocation",
        {"retval": {"type": "Z"}},
    )
    r(b"CLLocationManager", b"isAuthorizedForWidgetUpdates", {"retval": {"type": b"Z"}})
    r(
        b"CLLocationManager",
        b"isMonitoringAvailableForClass:",
        {"retval": {"type": b"Z"}},
    )
    r(b"CLLocationManager", b"isRangingAvailable", {"retval": {"type": b"Z"}})
    r(b"CLLocationManager", b"locationServicesEnabled", {"retval": {"type": "Z"}})
    r(
        b"CLLocationManager",
        b"pausesLocationUpdatesAutomatically",
        {"retval": {"type": b"Z"}},
    )
    r(b"CLLocationManager", b"regionMonitoringAvailable", {"retval": {"type": b"Z"}})
    r(b"CLLocationManager", b"regionMonitoringEnabled", {"retval": {"type": b"Z"}})
    r(
        b"CLLocationManager",
        b"requestTemporaryFullAccuracyAuthorizationWithPurposeKey:completion:",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    }
                }
            }
        },
    )
    r(
        b"CLLocationManager",
        b"requestTemporaryPreciseLocationAuthorizationWithPurposeKey:completion:",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    }
                }
            }
        },
    )
    r(
        b"CLLocationManager",
        b"setAllowsBackgroundLocationUpdates:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"CLLocationManager",
        b"setPausesLocationUpdatesAutomatically:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"CLLocationManager",
        b"setShowsBackgroundLocationIndicator:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"CLLocationManager",
        b"showsBackgroundLocationIndicator",
        {"retval": {"type": b"Z"}},
    )
    r(
        b"CLLocationManager",
        b"significantLocationChangeMonitoringAvailable",
        {"retval": {"type": b"Z"}},
    )
    r(
        b"CLLocationManager",
        b"startMonitoringLocationPushesWithCompletion:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"CLLocationSourceInformation",
        b"initWithSoftwareSimulationState:andExternalAccessoryState:",
        {"arguments": {2: {"type": b"Z"}, 3: {"type": b"Z"}}},
    )
    r(
        b"CLLocationSourceInformation",
        b"isProducedByAccessory",
        {"retval": {"type": b"Z"}},
    )
    r(
        b"CLLocationSourceInformation",
        b"isSimulatedBySoftware",
        {"retval": {"type": b"Z"}},
    )
    r(b"CLRegion", b"center", {"retval": {"type": b"{CLLocationCoordinate2D=dd}"}})
    r(
        b"CLRegion",
        b"containsCoordinate:",
        {
            "retval": {"type": b"Z"},
            "arguments": {2: {"type": b"{CLLocationCoordinate2D=dd}"}},
        },
    )
    r(
        b"CLRegion",
        b"initCircularRegionWithCenter:radius:identifier:",
        {"arguments": {2: {"type": b"{CLLocationCoordinate2D=dd}"}}},
    )
    r(b"CLRegion", b"notifyOnEntry", {"retval": {"type": b"Z"}})
    r(b"CLRegion", b"notifyOnExit", {"retval": {"type": b"Z"}})
    r(b"CLRegion", b"setNotifyOnEntry:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CLRegion", b"setNotifyOnExit:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"NSObject",
        b"didReceiveLocationPushPayload:completion:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {
                2: {"type": b"@"},
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}},
                    },
                    "type": b"@?",
                },
            },
        },
    )
    r(
        b"NSObject",
        b"locationManager:didChangeAuthorizationStatus:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"i"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:didDetermineState:forRegion:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"q"}, 4: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:didEnterRegion:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:didExitRegion:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:didFailRangingBeaconsForConstraint:error:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}, 4: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:didFailWithError:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:didFinishDeferredUpdatesWithError:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:didRangeBeacons:inRegion:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}, 4: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:didRangeBeacons:satisfyingConstraint:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}, 4: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:didStartMonitoringForRegion:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:didUpdateHeading:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:didUpdateLocations:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:didUpdateToLocation:fromLocation:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}, 4: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:didVisit:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:monitoringDidFailForRegion:withError:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}, 4: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManager:rangingBeaconsDidFailForRegion:withError:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}, 4: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"locationManagerDidChangeAuthorization:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"locationManagerDidPauseLocationUpdates:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"locationManagerDidResumeLocationUpdates:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"locationManagerShouldDisplayHeadingCalibration:",
        {"required": False, "retval": {"type": b"Z"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"serviceExtensionWillTerminate",
        {"required": False, "retval": {"type": b"v"}},
    )
finally:
    objc._updatingMetadata(False)
expressions = {}

# END OF FILE



import CoreLocation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCLLocation(TestCase):
    @min_os_level("10.6")
    def testStructs(self):
        v = CoreLocation.CLLocationCoordinate2D()
        self.assertIsInstance(v.latitude, float)
        self.assertIsInstance(v.longitude, float)
        self.assertPickleRoundTrips(v)
        self.assertEqual(
            CoreLocation.CLLocationCoordinate2D.__typestr__,
            b"{CLLocationCoordinate2D=dd}",
        )

    @min_os_level("10.6")
    def testConstants(self):
        self.assertIsInstance(CoreLocation.kCLDistanceFilterNone, float)

        self.assertIsInstance(CoreLocation.kCLLocationAccuracyBest, float)
        self.assertIsInstance(CoreLocation.kCLLocationAccuracyNearestTenMeters, float)
        self.assertIsInstance(CoreLocation.kCLLocationAccuracyHundredMeters, float)
        self.assertIsInstance(CoreLocation.kCLLocationAccuracyKilometer, float)
        self.assertIsInstance(CoreLocation.kCLLocationAccuracyThreeKilometers, float)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(CoreLocation.kCLLocationAccuracyBestForNavigation, float)
        self.assertIsInstance(
            CoreLocation.kCLLocationCoordinate2DInvalid,
            CoreLocation.CLLocationCoordinate2D,
        )

    @min_os_level("10.14")
    def testConstants10_14(self):
        self.assertIsInstance(CoreLocation.CLLocationDistanceMax, float)
        self.assertIsInstance(CoreLocation.CLTimeIntervalMax, float)

    @min_os_level("11.0")
    def testConstants11_0(self):
        self.assertIsInstance(CoreLocation.kCLLocationAccuracyReduced, float)

    @min_os_level("10.6")
    def testMethods(self):
        self.assertResultHasType(
            CoreLocation.CLLocation.coordinate,
            CoreLocation.CLLocationCoordinate2D.__typestr__,
        )
        # self.assertArgHasType(CoreLocation.CLLocation.setCoordinate_, 0, CoreLocation.CLLocationCoordinate2D.__typestr__)

        self.assertArgHasType(
            CoreLocation.CLLocation.initWithCoordinate_altitude_horizontalAccuracy_verticalAccuracy_timestamp_,
            0,
            CoreLocation.CLLocationCoordinate2D.__typestr__,
        )

    @min_os_level("12.0")
    def testMethods12_0(self):
        self.assertArgIsBOOL(
            CoreLocation.CLLocationSourceInformation.initWithSoftwareSimulationState_andExternalAccessoryState_,
            1,
        )

        self.assertResultIsBOOL(
            CoreLocation.CLLocationSourceInformation.isSimulatedBySoftware
        )
        self.assertResultIsBOOL(
            CoreLocation.CLLocationSourceInformation.isProducedByAccessory
        )

    @min_os_level("10.7")
    def testFunctions10_7(self):
        self.assertResultIsBOOL(CoreLocation.CLLocationCoordinate2DIsValid)
        self.assertFalse(
            CoreLocation.CLLocationCoordinate2DIsValid(
                CoreLocation.kCLLocationCoordinate2DInvalid
            )
        )

        loc = CoreLocation.CLLocationCoordinate2DMake(0.0, 0.0)
        self.assertIsInstance(loc, CoreLocation.CLLocationCoordinate2D)

        self.assertTrue(CoreLocation.CLLocationCoordinate2DIsValid(loc))



import CoreLocation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCLBeaconRegion(TestCase):
    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertResultIsBOOL(CoreLocation.CLBeaconRegion.notifyEntryStateOnDisplay)
        self.assertArgIsBOOL(
            CoreLocation.CLBeaconRegion.setNotifyEntryStateOnDisplay_, 0
        )
